-----------------------------------------------------------------------------------------------
-- Client Lua Script for NeedVsGreedLite
-- Original Lua and XML - Copyright (c) NCsoft. All rights reserved
-- Alterations made by Banpower and Randel (from Server Zhur) [IRL: Bertram Andr Nicolas, Kasper Myrtue]
-----------------------------------------------------------------------------------------------

require "Window"
require "Sound"

local NeedVsGreedLite = {}

local ktEvalColors = 
{
	[Item.CodeEnumItemQuality.Inferior] 		= ApolloColor.new("ItemQuality_Inferior"),
	[Item.CodeEnumItemQuality.Average] 			= ApolloColor.new("ItemQuality_Average"),
	[Item.CodeEnumItemQuality.Good] 			= ApolloColor.new("ItemQuality_Good"),
	[Item.CodeEnumItemQuality.Excellent] 		= ApolloColor.new("ItemQuality_Excellent"),
	[Item.CodeEnumItemQuality.Superb] 			= ApolloColor.new("ItemQuality_Superb"),
	[Item.CodeEnumItemQuality.Legendary] 		= ApolloColor.new("ItemQuality_Legendary"),
	[Item.CodeEnumItemQuality.Artifact]		 	= ApolloColor.new("ItemQuality_Artifact"),
}

function NeedVsGreedLite:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self
    return o
end

function NeedVsGreedLite:Init()
    Apollo.RegisterAddon(self)
	Apollo.RegisterSlashCommand("nvgl","SlashCommandFired",self)
	self.defaultSettings = { position = { left = 58,
										top = 500
										}
							}
	self.settings = self.defaultSettings
end

function NeedVsGreedLite:OnLoad()
	self.xmlDoc = XmlDoc.CreateFromFile("NeedVsGreedLite.xml")
	self.xmlDoc:RegisterCallback("OnDocumentReady", self) 

	Apollo.LoadSprites("Sprites.xml", "BarSprites")
end

function NeedVsGreedLite:OnDocumentReady()
	if  self.xmlDoc == nil then
		return
	end
	Apollo.RegisterEventHandler("LootRollUpdate",		"OnGroupLoot", self)
    Apollo.RegisterTimerHandler("WinnerCheckTimer", 	"OnOneSecTimer", self)
    Apollo.RegisterEventHandler("LootRollWon", 			"OnLootRollWon", self)
    Apollo.RegisterEventHandler("LootRollAllPassed", 	"OnLootRollAllPassed", self)
	
	Apollo.RegisterEventHandler("LootRollSelected", 	"OnLootRollSelected", self)
	Apollo.RegisterEventHandler("LootRollPassed", 		"OnLootRollPassed", self)
	Apollo.RegisterEventHandler("LootRoll", 			"OnLootRoll", self)

	--Apollo.RegisterEventHandler("GroupBagItemAdded", 	"OnGroupBagItemAdded", self) -- Appears deprecated

	Apollo.CreateTimer("WinnerCheckTimer", 1.0, false)
	Apollo.StopTimer("WinnerCheckTimer")
	self.wndMain = {}
	
	self.bTimerRunning = false
	self.tKnownLoot = nil
	self.tLootRolls = nil
	self.tMostRelevant = nil
	self.nLootNr = {}
	self.lootRolled = {}
	
	if GameLib.GetLootRolls() then
		self:OnGroupLoot()
	end
end

function NeedVsGreedLite:Close(wnd)
	if wnd then
		wnd:Destroy()
	end
end

function NeedVsGreedLite:OnSave(eType)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then
		return nil
	else
		return self.settings
	end
end

function NeedVsGreedLite:OnRestore(eType, tData)
	if eType == GameLib.CodeEnumAddonSaveLevel.Character then
		self.settings = tData
		end
end

-----------------------------------------------------------------------------------------------
-- Main Draw Method
-----------------------------------------------------------------------------------------------
function NeedVsGreedLite:OnGroupLoot()
	-- 
	maxTime = 0
	if not self.bTimerRunning then
		Apollo.StartTimer("WinnerCheckTimer")
		self.bTimerRunning = true
	end
end

function NeedVsGreedLite:UpdateKnownLoot()
	self.tLootRolls = GameLib.GetLootRolls()
	if not self.tLootRolls or #self.tLootRolls <= 0 then
		self.tKnownLoot = nil
		self.tLootRolls = nil
		self.tMostRelevant = nil
		return
	end
	self.tKnownLoot = {}
	for idx, tCurrentElement in ipairs(self.tLootRolls) do
		self.tKnownLoot[tCurrentElement.nLootId] = tCurrentElement
	end
	
	-- if self.tMostRelevant then
	  -- self.tMostRelevant = self.tKnownLoot[self.tMostRelevant.nLootId]
	-- end
	
	-- -- NOTE: self.tMostRelevant may have been set to nil above.
	-- if not self.tMostRelevant then
		-- for nLootId, tCurrentElement in pairs(self.tKnownLoot) do
			-- if not self.tMostRelevant or self.tMostRelevant.nTimeLeft > tCurrentElement.nTimeLeft then
				-- self.tMostRelevant = tCurrentElement
				-- --Print(math.floor(tCurrentElement.nTimeLeft / 1000))
			-- end
		-- end
	-- end
end

function NeedVsGreedLite:OnOneSecTimer()
	self:UpdateKnownLoot()
	if self.wndMain then
		for nLootId, wnd in pairs(self.wndMain) do
			if self.lootRolled[nLootId] and self.wndMain[nLootId] and self.wndMain[nLootId]:IsShown() then
				self.nLootNr[self.wndMain[nLootId]:GetData()] = nil
				self:Close(self.wndMain[nLootId])
			end
		end
	end
	
	if self.tLootRolls then
		for nLootId, tCurrentElement in pairs(self.tKnownLoot) do
			if not self.nLootNr[tCurrentElement.nLootId] then
				maxNr = 0
				if self.nLootNr then
					for is, itemNr in pairs(self.nLootNr) do
						if itemNr > maxNr then
							maxNr = itemNr
						end
					end
				end
				self.nLootNr[tCurrentElement.nLootId] = maxNr+1
			end
		end
		for nLootId, tCurrentElement in pairs(self.tKnownLoot) do
			if not self.tLootRolls and self.wndMain[tCurrentElement.nLootId] and self.wndMain[tCurrentElement.nLootId]:IsShown() then
				self:Close(self.wndMain[tCurrentElement.nLootId])
			end

			self:DrawLoot(tCurrentElement, #self.tLootRolls, self.nLootNr[tCurrentElement.nLootId])

			-- Art based on anchor
			if self.wndMain[tCurrentElement.nLootId] and self.wndMain[tCurrentElement.nLootId]:IsValid() then
				local nLeft, nTop, nRight, nBottom = self.wndMain[tCurrentElement.nLootId]:GetRect()
				if nLeft < 1 then
					self.wndMain[tCurrentElement.nLootId]:SetSprite("BK3:UI_BK3_Holo_Framing_2")
				else
					self.wndMain[tCurrentElement.nLootId]:SetSprite("BK3:UI_BK3_Holo_Framing_2")
				end
			end
		end
	end
	
	if self.tLootRolls and #self.tLootRolls > 0 then
		Apollo.StartTimer("WinnerCheckTimer")
	else
		self.bTimerRunning = false
	end
end

function NeedVsGreedLite:DrawLoot(tCurrentElement, nItemsInQueue, nItem)
	if not self.wndMain[tCurrentElement.nLootId] or not self.wndMain[tCurrentElement.nLootId]:IsValid() or not self.wndMain[tCurrentElement.nLootId]:GetData() then
		self:Close(self.wndMain[tCurrentElement.nLootId])
		self.wndMain[tCurrentElement.nLootId] = Apollo.LoadForm(self.xmlDoc, "NeedVsGreedLiteForm", nil, self)
		Sound.Play(Sound.PlayUIWindowNeedVsGreedOpen)
	end
	self.wndMain[tCurrentElement.nLootId]:SetData(tCurrentElement.nLootId)

	local itemCurrent = tCurrentElement.itemDrop
	local itemModData = tCurrentElement.tModData
	local tGlyphData = tCurrentElement.tSigilData
	self.wndMain[tCurrentElement.nLootId]:FindChild("LootTitle"):SetText(itemCurrent:GetName())
	self.wndMain[tCurrentElement.nLootId]:FindChild("LootTitle"):SetTextColor(ktEvalColors[itemCurrent:GetItemQuality()])
	self.wndMain[tCurrentElement.nLootId]:FindChild("ItemIcon"):SetSprite(itemCurrent:GetIcon())
	self:HelperBuildItemTooltip(self.wndMain[tCurrentElement.nLootId]:FindChild("ItemIcon"), itemCurrent, itemModData, tGlyphData)
	self.wndMain[tCurrentElement.nLootId]:SetAnchorPoints(0,0,0,0)
	self.wndMain[tCurrentElement.nLootId]:SetAnchorOffsets(self.settings.position.left,self.settings.position.top+(nItem-1)*36,self.settings.position.left+450,self.settings.position.top+32+(nItem-1)*36)
	self.wndMain[tCurrentElement.nLootId]:Show(true)
	--if nItemsInQueue > 1 then -- Do items in queue
	--	self.wndMain[tCurrentElement.nLootId]:FindChild("ItemsInQueueIcon"):SetTooltip(string.format("<P Font=\"CRB_InterfaceSmall\">%s</P>", String_GetWeaselString(Apollo.GetString("NeedVsGreedLite_NumItems"), nItemsInQueue)))
	--	self.wndMain[tCurrentElement.nLootId]:FindChild("ItemsInQueueText"):SetText(nItemsInQueue)
	--end
	--self.wndMain[tCurrentElement.nLootId]:FindChild("ItemsInQueueIcon"):Show(nItemsInQueue > 1)
	
	self.wndMain[tCurrentElement.nLootId]:FindChild("NeedBtn"):Show(GameLib.IsNeedRollAllowed(tCurrentElement.nLootId))
	self.wndMain[tCurrentElement.nLootId]:FindChild("NeedBtn"):Enable(GameLib.IsNeedRollAllowed(tCurrentElement.nLootId))

	-- TODO Timelimit
	local nTimeLeft = tCurrentElement.nTimeLeft
	if nTimeLeft > maxTime then
		maxTime = nTimeLeft
	end
	
	--self.wndMain:FindChild("TimeLeftText"):Show(true)
	--Event_FireGenericEvent("GenericEvent_LootChannelMessage", "Hello")
	if self.wndMain[tCurrentElement.nLootId] then
		self.wndbar = self.wndMain[tCurrentElement.nLootId]:FindChild("TimeLeftBar")
		self.wndbar:SetMax(maxTime)
		self.wndbar:SetProgress(nTimeLeft)
		NeedVsGreedLite:SetBarAppearance("TimeLeftBar", self.wndMain[tCurrentElement.nLootId], "Minimalist", ktEvalColors[itemCurrent:GetItemQuality()], 100)
	end
	if not nTimeLeft then
		Event_FireGenericEvent("GenericEvent_LootChannelMessage", "doesn't close")
		self.nLootNr[self.wndMain[tCurrentElement.nLootId]:GetData()] = nil
		self:Close(self.wndMain[tCurrentElement.nLootId])
	end
	-- local nTimeLeftSecs = nTimeLeft % 60
	-- local nTimeLeftMins = math.floor(nTimeLeft / 60)
	
	-- local strTimeLeft = tostring(nTimeLeftMins)
	-- if nTimeLeft < 0 then
		-- strTimeLeft = "0:00"
	-- elseif nTimeLeftSecs < 10 then
		-- strTimeLeft = strTimeLeft .. ":0" .. tostring(nTimeLeftSecs)
	-- else
		-- strTimeLeft = strTimeLeft .. ":" .. tostring(nTimeLeftSecs)
	-- end
	--self.wndMain:FindChild("TimeLeftText"):SetText(strTimeLeft)
end

-----------------------------------------------------------------------------------------------
-- Chat Message Events
-----------------------------------------------------------------------------------------------

function NeedVsGreedLite:OnLootRollAllPassed(itemLooted)
	-- Can be fired without self.wndMain
	ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_Loot, String_GetWeaselString(Apollo.GetString("NeedVsGreedLite_EveryonePassed"), itemLooted:GetName()))
end

function NeedVsGreedLite:OnLootRollWon(itemLooted, strWinner, bNeed)
	-- Can be fired without self.wndMain
	local strNeedOrGreed = "Greed"
	if bNeed then
		strNeedOrGreed = "Need"
	end

	-- Example Message: Alvin won Item Name (Need Roll).
	Event_FireGenericEvent("GenericEvent_LootChannelMessage", strWinner .. " won " .. itemLooted:GetName() .. " (" .. strNeedOrGreed .. " Roll)")
end

function NeedVsGreedLite:OnLootRollSelected(nLootItem, strPlayer, bNeed)

	local strNeedOrGreed = "Greed"
	if bNeed then
		strNeedOrGreed = "Need"
	end
	
	-- Example Message: strPlayer has selected to bNeed for nLootItem
	Event_FireGenericEvent("GenericEvent_LootChannelMessage", strPlayer .. " has selected to "  .. strNeedOrGreed .. " Roll for " .. nLootItem:GetName())
end

function NeedVsGreedLite:OnLootRollPassed(nLootItem, strPlayer)

	-- Example Message: strPlayer passed on nLootItem
	--Event_FireGenericEvent("GenericEvent_LootChannelMessage", strPlayer .. " passed on " nLootItem:GetName())
end

function NeedVsGreedLite:OnLootRoll(nLootItem, strPlayer, nRoll, bNeed)

	local strNeedOrGreed = "Greed"
	if bNeed then
		strNeedOrGreed = "Need"
	end
	-- Example String: strPlayer rolled nRoll for nLootItem (bNeed)
	Event_FireGenericEvent("GenericEvent_LootChannelMessage", strPlayer .. " rolled " .. tostring(nRoll) .. " for " .. nLootItem:GetName() .. " (" .. strNeedOrGreed .. " Roll)")
end

-----------------------------------------------------------------------------------------------
-- Buttons
-----------------------------------------------------------------------------------------------

function NeedVsGreedLite:OnNeedBtn(wndHandler, wndControl)
	GameLib.RollOnLoot(wndHandler:GetParent():GetParent():GetData(), true)
	self.nLootNr[wndHandler:GetParent():GetParent():GetData()] = nil
	self.lootRolled[wndHandler:GetParent():GetParent():GetData()] = true
	self:Close(wndHandler:GetParent():GetParent())
end

function NeedVsGreedLite:OnGreedBtn(wndHandler, wndControl)
	GameLib.RollOnLoot(wndHandler:GetParent():GetParent():GetData(), false)
	self.nLootNr[wndHandler:GetParent():GetParent():GetData()] = nil
	self.lootRolled[wndHandler:GetParent():GetParent():GetData()] = true
	self:Close(wndHandler:GetParent():GetParent())
end

function NeedVsGreedLite:OnPassBtn(wndHandler, wndControl)
	GameLib.PassOnLoot(wndHandler:GetParent():GetParent():GetData())
	self.nLootNr[wndHandler:GetParent():GetParent():GetData()] = nil
	self.lootRolled[wndHandler:GetParent():GetParent():GetData()] = true
	self:Close(wndHandler:GetParent():GetParent())
end

function NeedVsGreedLite:HelperBuildItemTooltip(wndArg, itemCurr, itemModData, tGlyphData)
	wndArg:SetTooltipDoc(nil)
	wndArg:SetTooltipDocSecondary(nil)
	local itemEquipped = itemCurr:GetEquippedItemForItemType()
	Tooltip.GetItemTooltipForm(self, wndArg, itemCurr, {bPrimary = true, bSelling = false, itemCompare = itemEquipped, itemModData = itemModData, tGlyphData = tGlyphData})
	--if itemEquipped then -- OLD
	--	Tooltip.GetItemTooltipForm(self, wndArg, itemEquipped, {bPrimary = false, bSelling = false, itemCompare = itemCurr})
	--end
end


-----------------------------------------------------------------------------------------------
-- Slash commands
-----------------------------------------------------------------------------------------------

function NeedVsGreedLite:SlashCommandFired(strCmd, strParam)

	if strParam == "" then
		ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_Command, "Please use one of the following commands:")
		ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_Command, "'/nvgl show' - to show the movable Rolls-Placement window")
		ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_Command, "'/nvgl hide' - to hide the movable Rolls-Placement window")
		--ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_Command, "'/nvgl options' - to show the options window") -- Commented this out till we work on an actual options window
		--
	elseif strParam == "show" or strParam == "unlock" then
		self.wndMove = Apollo.LoadForm(self.xmlDoc, "RollsPlacementFrame", nil, self)
		self.wndMove:SetAnchorPoints(0,0,0,0)
		self.wndMove:SetAnchorOffsets(self.settings.position.left, self.settings.position.top, self.settings.position.left + 450, self.settings.position.top + 176)
		self.wndMove:Show(true)
		--ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_Command, "Show the 'RollsPlacementFrame' window") --TODO:Show the "RollsPlacementFrame" window
		--
	elseif strParam == "hide" or strParam == "lock" then
		if self.wndMove:IsVisible() then
			local left, top, right, bottom = self.wndMove:GetAnchorOffsets()
			self.settings.position.left = left
			self.settings.position.top = top
			self.wndMove:Show(false)
		end
		--ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_Command, "Hide the 'RollsPlacementFrame' window") --TODO:Hide and save the "RollsPlacementFrame" window
		--
	elseif strParam == "reset" then
		self.settings = self.defaultSettings
		ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_Command, "Your settings have been reset back to default")
		--
	elseif strParam == "options" or strParam == "option" or strParam == "config" then
		--Nothing to see here
	end
end

-----------------------------------------------------------------------------------------------
-- Utility functions
-----------------------------------------------------------------------------------------------

function NeedVsGreedLite:SetBarAppearance(bar, barWnd, texture, color, percentTrans) 
    --Consider: input from Colorpicker will be at least 6 hex, possibly 8 hex. You can append 2 hex transparency with slider value stored to percentTrans.
    
    local transHex = "ff"
    local colorStr = "pink"
    
    if percentTrans and type(percentTrans) == "number" then
    	transHex = string.format("%x", (percentTrans/100)*255)
    end
    
    -- if tonumber("0x"..color) then
    	-- if string.len(color) == 6 then
    	    -- colorStr = transHex .. color
    	-- else
    	    -- colorStr = color
    	-- end
    -- else
    	-- colorStr = color
    -- end  	

    --Set Bar Color/transparency
	if texture == "TrueFrost" then
    	barWnd:FindChild(bar):SetBarColor("FFFFFFFF")
	else
    	barWnd:FindChild(bar):SetBarColor(color)
	end
    
    --Set Bar Texture
    barWnd:FindChild(bar):SetFillSprite("BarSprites:" .. texture)
    barWnd:FindChild(bar):SetFullSprite("BarSprites:" .. texture)
end


local NeedVsGreedLiteInst = NeedVsGreedLite:new()
NeedVsGreedLiteInst:Init()
